@@ -131,11 +131,9 @@ CONNECTION_NAME = 'dmypy'  # type: Final
 
 
 def process_start_options(flags: List[str], allow_sources: bool) -> Options:
-    sources, options = mypy.main.process_options(['-i'] + flags,
-                                                 require_targets=False,
-                                                 server_options=True)
-    if sources and not allow_sources:
-        sys.exit("dmypy: start/restart does not accept sources")
+    _, options = mypy.main.process_options(
+        ['-i'] + flags, require_targets=False, server_options=True
+    )
     if options.report_dirs:
         sys.exit("dmypy: start/restart cannot generate reports")
     if options.junit_xml:
@@ -275,7 +273,7 @@ class Server:
         res.update(get_meminfo())
         if fswatcher_dump_file:
             data = self.fswatcher.dump_file_data() if hasattr(self, 'fswatcher') else {}
-            # Using .dumps and then writing was noticably faster than using dump
+            # Using .dumps and then writing was noticeably faster than using dump
             s = json.dumps(data)
             with open(fswatcher_dump_file, 'w') as f:
                 f.write(s)
@@ -315,9 +313,9 @@ class Server:
             if self.fine_grained_manager:
                 manager = self.fine_grained_manager.manager
                 start_plugins_snapshot = manager.plugins_snapshot
-                _, current_plugins_snapshot = mypy.build.load_plugins(options,
-                                                                      manager.errors,
-                                                                      sys.stdout)
+                _, current_plugins_snapshot = mypy.build.load_plugins(
+                    options, manager.errors, sys.stdout, extra_plugins=()
+                )
                 if current_plugins_snapshot != start_plugins_snapshot:
                     return {'restart': 'plugins changed'}
         except InvalidSourceList as err:
@@ -425,7 +423,7 @@ class Server:
                 assert state.path is not None
                 self.fswatcher.set_file_data(
                     state.path,
-                    FileData(st_mtime=float(meta.mtime), st_size=meta.size, md5=meta.hash))
+                    FileData(st_mtime=float(meta.mtime), st_size=meta.size, hash=meta.hash))
 
             changed, removed = self.find_changed(sources)
 
@@ -566,7 +564,9 @@ class Server:
                     **kwargs: Any) -> Dict[str, object]:
         """Suggest a signature for a function."""
         if not self.fine_grained_manager:
-            return {'error': "Command 'suggest' is only valid after a 'check' command"}
+            return {
+                'error': "Command 'suggest' is only valid after a 'check' command"
+                " (that produces no parse errors)"}
         engine = SuggestionEngine(self.fine_grained_manager, **kwargs)
         try:
             if callsites:
