@@ -1,13 +1,14 @@
 """Mypy type checker command line tool."""
 
 import argparse
+from gettext import gettext
 import os
 import subprocess
 import sys
 import time
 
-from typing import Any, Dict, List, Optional, Tuple, TextIO
-from typing_extensions import Final
+from typing import Any, Dict, IO, List, Optional, Sequence, Tuple, TextIO, Union
+from typing_extensions import Final, NoReturn
 
 from mypy import build
 from mypy import defaults
@@ -252,6 +253,99 @@ FOOTER = """Environment variables:
   Define MYPY_CACHE_DIR to override configuration cache_dir path."""  # type: Final
 
 
+class CapturableArgumentParser(argparse.ArgumentParser):
+
+    """Override ArgumentParser methods that use sys.stdout/sys.stderr directly.
+
+    This is needed because hijacking sys.std* is not thread-safe,
+    yet output must be captured to properly support mypy.api.run.
+    """
+
+    def __init__(self, *args: Any, **kwargs: Any):
+        self.stdout = kwargs.pop('stdout', sys.stdout)
+        self.stderr = kwargs.pop('stderr', sys.stderr)
+        super().__init__(*args, **kwargs)
+
+    # =====================
+    # Help-printing methods
+    # =====================
+    def print_usage(self, file: Optional[IO[str]] = None) -> None:
+        if file is None:
+            file = self.stdout
+        self._print_message(self.format_usage(), file)
+
+    def print_help(self, file: Optional[IO[str]] = None) -> None:
+        if file is None:
+            file = self.stdout
+        self._print_message(self.format_help(), file)
+
+    def _print_message(self, message: str, file: Optional[IO[str]] = None) -> None:
+        if message:
+            if file is None:
+                file = self.stderr
+            file.write(message)
+
+    # ===============
+    # Exiting methods
+    # ===============
+    def exit(self, status: int = 0, message: Optional[str] = None) -> NoReturn:
+        if message:
+            self._print_message(message, self.stderr)
+        sys.exit(status)
+
+    def error(self, message: str) -> NoReturn:
+        """error(message: string)
+
+        Prints a usage message incorporating the message to stderr and
+        exits.
+
+        If you override this in a subclass, it should not return -- it
+        should either exit or raise an exception.
+        """
+        self.print_usage(self.stderr)
+        args = {'prog': self.prog, 'message': message}
+        self.exit(2, gettext('%(prog)s: error: %(message)s\n') % args)
+
+
+class CapturableVersionAction(argparse.Action):
+
+    """Supplement CapturableArgumentParser to handle --version.
+
+    This is nearly identical to argparse._VersionAction except,
+    like CapturableArgumentParser, it allows output to be captured.
+
+    Another notable difference is that version is mandatory.
+    This allows removing a line in __call__ that falls back to parser.version
+    (which does not appear to exist).
+    """
+
+    def __init__(self,
+                 option_strings: Sequence[str],
+                 version: str,
+                 dest: str = argparse.SUPPRESS,
+                 default: str = argparse.SUPPRESS,
+                 help: str = "show program's version number and exit",
+                 stdout: Optional[IO[str]] = None):
+        super().__init__(
+            option_strings=option_strings,
+            dest=dest,
+            default=default,
+            nargs=0,
+            help=help)
+        self.version = version
+        self.stdout = stdout or sys.stdout
+
+    def __call__(self,
+                 parser: argparse.ArgumentParser,
+                 namespace: argparse.Namespace,
+                 values: Union[str, Sequence[Any], None],
+                 option_string: Optional[str] = None) -> NoReturn:
+        formatter = parser._get_formatter()
+        formatter.add_text(self.version)
+        parser._print_message(formatter.format_help(), self.stdout)
+        parser.exit()
+
+
 def process_options(args: List[str],
                     stdout: Optional[TextIO] = None,
                     stderr: Optional[TextIO] = None,
@@ -269,13 +363,15 @@ def process_options(args: List[str],
     stdout = stdout or sys.stdout
     stderr = stderr or sys.stderr
 
-    parser = argparse.ArgumentParser(prog=program,
-                                     usage=header,
-                                     description=DESCRIPTION,
-                                     epilog=FOOTER,
-                                     fromfile_prefix_chars='@',
-                                     formatter_class=AugmentedHelpFormatter,
-                                     add_help=False)
+    parser = CapturableArgumentParser(prog=program,
+                                      usage=header,
+                                      description=DESCRIPTION,
+                                      epilog=FOOTER,
+                                      fromfile_prefix_chars='@',
+                                      formatter_class=AugmentedHelpFormatter,
+                                      add_help=False,
+                                      stdout=stdout,
+                                      stderr=stderr)
 
     strict_flag_names = []  # type: List[str]
     strict_flag_assignments = []  # type: List[Tuple[str, bool]]
@@ -328,9 +424,10 @@ def process_options(args: List[str],
         '-v', '--verbose', action='count', dest='verbosity',
         help="More verbose messages")
     general_group.add_argument(
-        '-V', '--version', action='version',
+        '-V', '--version', action=CapturableVersionAction,
         version='%(prog)s ' + __version__,
-        help="Show program's version number and exit")
+        help="Show program's version number and exit",
+        stdout=stdout)
 
     config_group = parser.add_argument_group(
         title='Config file',
@@ -348,6 +445,10 @@ def process_options(args: List[str],
     imports_group = parser.add_argument_group(
         title='Import discovery',
         description="Configure how imports are discovered and followed.")
+    add_invertible_flag(
+        '--namespace-packages', default=False,
+        help="Support namespace packages (PEP 420, __init__.py-less)",
+        group=imports_group)
     imports_group.add_argument(
         '--ignore-missing-imports', action='store_true',
         help="Silently ignore imports of missing modules")
@@ -366,10 +467,6 @@ def process_options(args: List[str],
     imports_group.add_argument(
         '--no-silence-site-packages', action='store_true',
         help="Do not silence errors in PEP 561 compliant installed packages")
-    add_invertible_flag(
-        '--namespace-packages', default=False,
-        help="Support namespace packages (PEP 420, __init__.py-less)",
-        group=imports_group)
 
     platform_group = parser.add_argument_group(
         title='Platform configuration',
@@ -397,14 +494,11 @@ def process_options(args: List[str],
         help="Additional variable to be considered False (may be repeated)")
 
     disallow_any_group = parser.add_argument_group(
-        title='Dynamic typing',
+        title='Disallow dynamic typing',
         description="Disallow the use of the dynamic 'Any' type under certain conditions.")
     disallow_any_group.add_argument(
         '--disallow-any-unimported', default=False, action='store_true',
         help="Disallow Any types resulting from unfollowed imports")
-    add_invertible_flag('--disallow-subclassing-any', default=False, strict_flag=True,
-                        help="Disallow subclassing values of type 'Any' when defining classes",
-                        group=disallow_any_group)
     disallow_any_group.add_argument(
         '--disallow-any-expr', default=False, action='store_true',
         help='Disallow all expressions that have type Any')
@@ -418,6 +512,9 @@ def process_options(args: List[str],
     add_invertible_flag('--disallow-any-generics', default=False, strict_flag=True,
                         help='Disallow usage of generic types that do not specify explicit type '
                         'parameters', group=disallow_any_group)
+    add_invertible_flag('--disallow-subclassing-any', default=False, strict_flag=True,
+                        help="Disallow subclassing values of type 'Any' when defining classes",
+                        group=disallow_any_group)
 
     untyped_group = parser.add_argument_group(
         title='Untyped definitions and calls',
@@ -459,12 +556,10 @@ def process_options(args: List[str],
         help="Disable strict Optional checks (inverse: --strict-optional)")
     none_group.add_argument(
         '--strict-optional-whitelist', metavar='GLOB', nargs='*',
-        help="Suppress strict Optional errors in all but the provided files; "
-             "implies --strict-optional (may suppress certain other errors "
-             "in non-whitelisted files)")
+        help=argparse.SUPPRESS)
 
     lint_group = parser.add_argument_group(
-        title='Warnings',
+        title='Configuring warnings',
         description="Detect code that is sound but redundant or problematic.")
     add_invertible_flag('--warn-redundant-casts', default=False, strict_flag=True,
                         help="Warn about casting an expression to its inferred type",
@@ -492,7 +587,7 @@ def process_options(args: List[str],
     # We add `--strict` near the end so we don't accidentally miss any strictness
     # flags that are added after this group.
     strictness_group = parser.add_argument_group(
-        title='Other strictness checks')
+        title='Miscellaneous strictness flags')
 
     add_invertible_flag('--allow-untyped-globals', default=False, strict_flag=False,
                         help="Suppress toplevel errors caused by missing annotations",
@@ -502,16 +597,50 @@ def process_options(args: List[str],
                         help="Allow unconditional variable redefinition with a new type",
                         group=strictness_group)
 
-    add_invertible_flag('--strict-equality', default=False, strict_flag=False,
-                        help="Prohibit equality, identity, and container checks for"
-                             " non-overlapping types",
-                        group=strictness_group)
-
     add_invertible_flag('--no-implicit-reexport', default=True, strict_flag=True,
                         dest='implicit_reexport',
                         help="Treat imports as private unless aliased",
                         group=strictness_group)
 
+    add_invertible_flag('--strict-equality', default=False, strict_flag=True,
+                        help="Prohibit equality, identity, and container checks for"
+                             " non-overlapping types",
+                        group=strictness_group)
+
+    strict_help = "Strict mode; enables the following flags: {}".format(
+        ", ".join(strict_flag_names))
+    strictness_group.add_argument(
+        '--strict', action='store_true', dest='special-opts:strict',
+        help=strict_help)
+
+    error_group = parser.add_argument_group(
+        title='Configuring error messages',
+        description="Adjust the amount of detail shown in error messages.")
+    add_invertible_flag('--show-error-context', default=False,
+                        dest='show_error_context',
+                        help='Precede errors with "note:" messages explaining context',
+                        group=error_group)
+    add_invertible_flag('--show-column-numbers', default=False,
+                        help="Show column numbers in error messages",
+                        group=error_group)
+    add_invertible_flag('--show-error-codes', default=False,
+                        help="Show error codes in error messages",
+                        group=error_group)
+    add_invertible_flag('--pretty', default=False,
+                        help="Use visually nicer output in error messages:"
+                             " Use soft word wrap, show source code snippets,"
+                             " and show error location markers",
+                        group=error_group)
+    add_invertible_flag('--no-color-output', dest='color_output', default=True,
+                        help="Do not colorize error messages",
+                        group=error_group)
+    add_invertible_flag('--no-error-summary', dest='error_summary', default=True,
+                        help="Do not show error stats summary",
+                        group=error_group)
+    add_invertible_flag('--show-absolute-path', default=False,
+                        help="Show absolute paths to files",
+                        group=error_group)
+
     incremental_group = parser.add_argument_group(
         title='Incremental mode',
         description="Adjust how mypy incrementally type checks and caches modules. "
@@ -543,7 +672,7 @@ def process_options(args: List[str],
         help="Skip cache internal consistency checks based on mtime")
 
     internals_group = parser.add_argument_group(
-        title='Mypy internals',
+        title='Advanced options',
         description="Debug and customize mypy internals.")
     internals_group.add_argument(
         '--pdb', action='store_true', help="Invoke pdb on fatal error")
@@ -554,7 +683,7 @@ def process_options(args: List[str],
         '--raise-exceptions', action='store_true', help="Raise exception on fatal error"
     )
     internals_group.add_argument(
-        '--custom-typing', metavar='MODULE', dest='custom_typing_module',
+        '--custom-typing-module', metavar='MODULE', dest='custom_typing_module',
         help="Use a custom typing module")
     internals_group.add_argument(
         '--custom-typeshed-dir', metavar='DIR',
@@ -571,44 +700,14 @@ def process_options(args: List[str],
     add_invertible_flag('--fast-exit', default=False, help=argparse.SUPPRESS,
                         group=internals_group)
 
-    error_group = parser.add_argument_group(
-        title='Error reporting',
-        description="Adjust the amount of detail shown in error messages.")
-    add_invertible_flag('--show-error-context', default=False,
-                        dest='show_error_context',
-                        help='Precede errors with "note:" messages explaining context',
-                        group=error_group)
-    add_invertible_flag('--show-column-numbers', default=False,
-                        help="Show column numbers in error messages",
-                        group=error_group)
-    add_invertible_flag('--show-error-codes', default=False,
-                        help="Show error codes in error messages",
-                        group=error_group)
-    add_invertible_flag('--pretty', default=False,
-                        help="Use visually nicer output in error messages:"
-                             " Use soft word wrap, show source code snippets,"
-                             " and error location markers",
-                        group=error_group)
-    add_invertible_flag('--no-color-output', dest='color_output', default=True,
-                        help="Do not colorize error messages",
-                        group=error_group)
-    add_invertible_flag('--no-error-summary', dest='error_summary', default=True,
-                        help="Do not show error stats summary",
-                        group=error_group)
-
-    strict_help = "Strict mode; enables the following flags: {}".format(
-        ", ".join(strict_flag_names))
-    strictness_group.add_argument(
-        '--strict', action='store_true', dest='special-opts:strict',
-        help=strict_help)
-
     report_group = parser.add_argument_group(
         title='Report generation',
         description='Generate a report in the specified format.')
     for report_type in sorted(defaults.REPORTER_NAMES):
-        report_group.add_argument('--%s-report' % report_type.replace('_', '-'),
-                                  metavar='DIR',
-                                  dest='special-opts:%s_report' % report_type)
+        if report_type not in {'memory-xml'}:
+            report_group.add_argument('--%s-report' % report_type.replace('_', '-'),
+                                      metavar='DIR',
+                                      dest='special-opts:%s_report' % report_type)
 
     other_group = parser.add_argument_group(
         title='Miscellaneous')
@@ -616,13 +715,13 @@ def process_options(args: List[str],
         '--quickstart-file', help=argparse.SUPPRESS)
     other_group.add_argument(
         '--junit-xml', help="Write junit.xml to the given file")
-    other_group.add_argument(
-        '--scripts-are-modules', action='store_true',
-        help="Script x becomes module x instead of __main__")
     other_group.add_argument(
         '--find-occurrences', metavar='CLASS.MEMBER',
         dest='special-opts:find_occurrences',
         help="Print out all usages of a class member (experimental)")
+    other_group.add_argument(
+        '--scripts-are-modules', action='store_true',
+        help="Script x becomes module x instead of __main__")
 
     if server_options:
         # TODO: This flag is superfluous; remove after a short transition (2018-03-16)
@@ -639,6 +738,9 @@ def process_options(args: List[str],
     parser.add_argument(
         '--inferstats', action='store_true', dest='dump_inference_stats',
         help=argparse.SUPPRESS)
+    parser.add_argument(
+        '--dump-build-stats', action='store_true',
+        help=argparse.SUPPRESS)
     # --debug-cache will disable any cache-related compressions/optimizations,
     # which will make the cache writing process output pretty-printed JSON (which
     # is easier to debug).
@@ -704,15 +806,19 @@ def process_options(args: List[str],
     if config_file and not os.path.exists(config_file):
         parser.error("Cannot find config file '%s'" % config_file)
 
-    # Parse config file first, so command line can override.
     options = Options()
-    parse_config_file(options, config_file, stdout, stderr)
+
+    def set_strict_flags() -> None:
+        for dest, value in strict_flag_assignments:
+            setattr(options, dest, value)
+
+    # Parse config file first, so command line can override.
+    parse_config_file(options, set_strict_flags, config_file, stdout, stderr)
 
     # Set strict flags before parsing (if strict mode enabled), so other command
     # line options can override.
     if getattr(dummy, 'special-opts:strict'):  # noqa
-        for dest, value in strict_flag_assignments:
-            setattr(options, dest, value)
+        set_strict_flags()
 
     # Override cache_dir if provided in the environment
     environ_cache_dir = os.getenv('MYPY_CACHE_DIR', '')
