@@ -5,12 +5,14 @@ from typing_extensions import Final
 
 from mypy.nodes import (
     ARG_OPT, ARG_POS, MDEF, Argument, AssignmentStmt, CallExpr,
-    Context, Expression, FuncDef, JsonDict, NameExpr, RefExpr,
+    Context, Expression, JsonDict, NameExpr, RefExpr,
     SymbolTableNode, TempNode, TypeInfo, Var, TypeVarExpr, PlaceholderNode
 )
-from mypy.plugin import ClassDefContext
-from mypy.plugins.common import add_method, _get_decorator_bool_argument
-from mypy.types import Instance, NoneType, TypeVarDef, TypeVarType, get_proper_type
+from mypy.plugin import ClassDefContext, SemanticAnalyzerPluginInterface
+from mypy.plugins.common import (
+    add_method, _get_decorator_bool_argument, deserialize_and_fixup_type,
+)
+from mypy.types import Type, Instance, NoneType, TypeVarDef, TypeVarType, get_proper_type
 from mypy.server.trigger import make_wildcard_trigger
 
 # The set of decorators that generate dataclasses.
@@ -31,6 +33,7 @@ class DataclassAttribute:
             has_default: bool,
             line: int,
             column: int,
+            type: Optional[Type],
     ) -> None:
         self.name = name
         self.is_in_init = is_in_init
@@ -38,19 +41,21 @@ class DataclassAttribute:
         self.has_default = has_default
         self.line = line
         self.column = column
+        self.type = type
 
-    def to_argument(self, info: TypeInfo) -> Argument:
+    def to_argument(self) -> Argument:
         return Argument(
-            variable=self.to_var(info),
-            type_annotation=info[self.name].type,
+            variable=self.to_var(),
+            type_annotation=self.type,
             initializer=None,
             kind=ARG_OPT if self.has_default else ARG_POS,
         )
 
-    def to_var(self, info: TypeInfo) -> Var:
-        return Var(self.name, info[self.name].type)
+    def to_var(self) -> Var:
+        return Var(self.name, self.type)
 
     def serialize(self) -> JsonDict:
+        assert self.type
         return {
             'name': self.name,
             'is_in_init': self.is_in_init,
@@ -58,11 +63,16 @@ class DataclassAttribute:
             'has_default': self.has_default,
             'line': self.line,
             'column': self.column,
+            'type': self.type.serialize(),
         }
 
     @classmethod
-    def deserialize(cls, info: TypeInfo, data: JsonDict) -> 'DataclassAttribute':
-        return cls(**data)
+    def deserialize(
+        cls, info: TypeInfo, data: JsonDict, api: SemanticAnalyzerPluginInterface
+    ) -> 'DataclassAttribute':
+        data = data.copy()
+        typ = deserialize_and_fixup_type(data.pop('type'), api)
+        return cls(type=typ, **data)
 
 
 class DataclassTransformer:
@@ -81,12 +91,7 @@ class DataclassTransformer:
             # Some definitions are not ready, defer() should be already called.
             return
         for attr in attributes:
-            node = info.get(attr.name)
-            if node is None:
-                # Nodes of superclass InitVars not used in __init__ cannot be reached.
-                assert attr.is_init_var and not attr.is_in_init
-                continue
-            if node.type is None:
+            if attr.type is None:
                 ctx.api.defer()
                 return
         decorator_arguments = {
@@ -96,7 +101,7 @@ class DataclassTransformer:
             'frozen': _get_decorator_bool_argument(self._ctx, 'frozen', False),
         }
 
-        # If there are no attributes, it may be that the new semantic analyzer has not
+        # If there are no attributes, it may be that the semantic analyzer has not
         # processed them yet. In order to work around this, we can simply skip generating
         # __init__ if there are no attributes, because if the user truly did not define any,
         # then the object default __init__ with an empty signature will be present anyway.
@@ -106,7 +111,7 @@ class DataclassTransformer:
             add_method(
                 ctx,
                 '__init__',
-                args=[attr.to_argument(info) for attr in attributes if attr.is_in_init],
+                args=[attr.to_argument() for attr in attributes if attr.is_in_init],
                 return_type=NoneType(),
             )
 
@@ -114,7 +119,7 @@ class DataclassTransformer:
                 decorator_arguments['order']):
             # Type variable for self types in generated methods.
             obj_type = ctx.api.named_type('__builtins__.object')
-            self_tvar_expr = TypeVarExpr(SELF_TVAR_NAME, info.fullname() + '.' + SELF_TVAR_NAME,
+            self_tvar_expr = TypeVarExpr(SELF_TVAR_NAME, info.fullname + '.' + SELF_TVAR_NAME,
                                          [], obj_type)
             info.names[SELF_TVAR_NAME] = SymbolTableNode(MDEF, self_tvar_expr)
 
@@ -125,7 +130,7 @@ class DataclassTransformer:
                 # the same type as self (covariant).  Note the
                 # "self_type" parameter to add_method.
                 obj_type = ctx.api.named_type('__builtins__.object')
-                cmp_tvar_def = TypeVarDef(SELF_TVAR_NAME, info.fullname() + '.' + SELF_TVAR_NAME,
+                cmp_tvar_def = TypeVarDef(SELF_TVAR_NAME, info.fullname + '.' + SELF_TVAR_NAME,
                                           -1, [], obj_type)
                 cmp_other_type = TypeVarType(cmp_tvar_def)
                 cmp_return_type = ctx.api.named_type('__builtins__.bool')
@@ -148,7 +153,7 @@ class DataclassTransformer:
                 # Like for __eq__ and __ne__, we want "other" to match
                 # the self type.
                 obj_type = ctx.api.named_type('__builtins__.object')
-                order_tvar_def = TypeVarDef(SELF_TVAR_NAME, info.fullname() + '.' + SELF_TVAR_NAME,
+                order_tvar_def = TypeVarDef(SELF_TVAR_NAME, info.fullname + '.' + SELF_TVAR_NAME,
                                             -1, [], obj_type)
                 order_other_type = TypeVarType(order_tvar_def)
                 order_return_type = ctx.api.named_type('__builtins__.bool')
@@ -191,7 +196,7 @@ class DataclassTransformer:
                     del info.names[attr.name]
                 else:
                     # Nodes of superclass InitVars not used in __init__ cannot be reached.
-                    assert attr.is_init_var and not attr.is_in_init
+                    assert attr.is_init_var
                 for stmt in info.defn.defs.body:
                     if isinstance(stmt, AssignmentStmt) and stmt.unanalyzed_type:
                         lvalue = stmt.lvalues[0]
@@ -247,7 +252,7 @@ class DataclassTransformer:
             is_init_var = False
             node_type = get_proper_type(node.type)
             if (isinstance(node_type, Instance) and
-                    node_type.type.fullname() == 'dataclasses.InitVar'):
+                    node_type.type.fullname == 'dataclasses.InitVar'):
                 is_init_var = True
                 node.type = node_type.args[0]
 
@@ -269,6 +274,11 @@ class DataclassTransformer:
             elif not isinstance(stmt.rvalue, TempNode):
                 has_default = True
 
+            if not has_default:
+                # Make all non-default attributes implicit because they are de-facto set
+                # on self in the generated __init__(), not in the class body.
+                sym.implicit = True
+
             known_attrs.add(lhs.name)
             attrs.append(DataclassAttribute(
                 name=lhs.name,
@@ -277,6 +287,7 @@ class DataclassTransformer:
                 has_default=has_default,
                 line=stmt.line,
                 column=stmt.column,
+                type=sym.type,
             ))
 
         # Next, collect attributes belonging to any class in the MRO
@@ -285,39 +296,30 @@ class DataclassTransformer:
         # copy() because we potentially modify all_attrs below and if this code requires debugging
         # we'll have unmodified attrs laying around.
         all_attrs = attrs.copy()
-        init_method = cls.info.get_method('__init__')
         for info in cls.info.mro[1:-1]:
             if 'dataclass' not in info.metadata:
                 continue
 
             super_attrs = []
             # Each class depends on the set of attributes in its dataclass ancestors.
-            ctx.api.add_plugin_dependency(make_wildcard_trigger(info.fullname()))
+            ctx.api.add_plugin_dependency(make_wildcard_trigger(info.fullname))
 
             for data in info.metadata['dataclass']['attributes']:
                 name = data['name']  # type: str
                 if name not in known_attrs:
-                    attr = DataclassAttribute.deserialize(info, data)
-                    if attr.is_init_var and isinstance(init_method, FuncDef):
-                        # InitVars are removed from classes so, in order for them to be inherited
-                        # properly, we need to re-inject them into subclasses' sym tables here.
-                        # To do that, we look 'em up from the parents' __init__.  These variables
-                        # are subsequently removed from the sym table at the end of
-                        # DataclassTransformer.transform.
-                        for arg, arg_name in zip(init_method.arguments, init_method.arg_names):
-                            if arg_name == attr.name:
-                                cls.info.names[attr.name] = SymbolTableNode(MDEF, arg.variable)
-
+                    attr = DataclassAttribute.deserialize(info, data, ctx.api)
                     known_attrs.add(name)
                     super_attrs.append(attr)
-                else:
+                elif all_attrs:
                     # How early in the attribute list an attribute appears is determined by the
                     # reverse MRO, not simply MRO.
                     # See https://docs.python.org/3/library/dataclasses.html#inheritance for
                     # details.
-                    (attr,) = [a for a in all_attrs if a.name == name]
-                    all_attrs.remove(attr)
-                    super_attrs.append(attr)
+                    for attr in all_attrs:
+                        if attr.name == name:
+                            all_attrs.remove(attr)
+                            super_attrs.append(attr)
+                            break
             all_attrs = super_attrs + all_attrs
 
         # Ensure that arguments without a default don't follow
@@ -353,11 +355,11 @@ class DataclassTransformer:
                 assert isinstance(var, Var)
                 var.is_property = True
             else:
-                var = attr.to_var(info)
+                var = attr.to_var()
                 var.info = info
                 var.is_property = True
-                var._fullname = info.fullname() + '.' + var.name()
-                info.names[var.name()] = SymbolTableNode(MDEF, var)
+                var._fullname = info.fullname + '.' + var.name
+                info.names[var.name] = SymbolTableNode(MDEF, var)
 
 
 def dataclass_class_maker_callback(ctx: ClassDefContext) -> None:
