@@ -1,6 +1,4 @@
-MYPY = False
-if MYPY:
-    from typing_extensions import Final
+from typing_extensions import Final
 
 PREFIX = 'CPyPy_'  # type: Final # Python wrappers
 NATIVE_PREFIX = 'CPyDef_'  # type: Final # Native functions etc.
@@ -31,3 +29,11 @@ FAST_ISINSTANCE_MAX_SUBCLASSES = 2  # type: Final
 
 def decorator_helper_name(func_name: str) -> str:
     return '__mypyc_{}_decorator_helper__'.format(func_name)
+
+
+def shared_lib_name(group_name: str) -> str:
+    """Given a group name, return the actual name of its extension module.
+
+    (This just adds a suffix to the final component.)
+    """
+    return '{}__mypyc'.format(group_name)
