@@ -4,7 +4,6 @@
 # single module and it should be renamed.
 
 import os
-import hashlib
 import json
 from collections import OrderedDict
 from typing import List, Tuple, Dict, Iterable, Set, TypeVar, Optional
@@ -18,8 +17,11 @@ from mypy.errors import CompileError
 from mypy.options import Options
 from mypy.plugin import Plugin, ReportConfigContext
 from mypy.fscache import FileSystemCache
+from mypy.util import hash_digest
 
-from mypyc import genops
+from mypyc import genopsmain
+from mypyc.genopsprepare import load_type_map
+from mypyc.genopsmapper import Mapper
 from mypyc.common import (
     PREFIX, TOP_LEVEL_NAME, INT_PREFIX, MODULE_PREFIX, shared_lib_name,
 )
@@ -53,7 +55,7 @@ from mypyc.errors import Errors
 # modules: one shim per module and one shared library containing all
 # the actual code.
 # In fully separate compilation, we (unfortunately) will generate 2*N
-# extension modules: one shim per module and also one library containg
+# extension modules: one shim per module and also one library containing
 # each module's actual code. (This might be fixable in the future,
 # but allows a clean separation between setup of the export tables
 # (see generate_export_table) and running module top levels.)
@@ -144,7 +146,7 @@ class MypycPlugin(Plugin):
                     contents = f.read()
             except FileNotFoundError:
                 return None
-            real_hash = hashlib.md5(contents).hexdigest()
+            real_hash = hash_digest(contents)
             if hash != real_hash:
                 return None
 
@@ -177,7 +179,7 @@ def parse_and_typecheck(
 def compile_scc_to_ir(
     scc: List[MypyFile],
     result: BuildResult,
-    mapper: genops.Mapper,
+    mapper: Mapper,
     compiler_options: CompilerOptions,
     errors: Errors,
 ) -> ModuleIRs:
@@ -200,7 +202,7 @@ def compile_scc_to_ir(
         print("Compiling {}".format(", ".join(x.name for x in scc)))
 
     # Generate basic IR, with missing exception and refcount handling.
-    modules = genops.build_ir(
+    modules = genopsmain.build_ir(
         scc, result.graph, result.types, mapper, compiler_options, errors
     )
     if errors.num_errors > 0:
@@ -224,7 +226,7 @@ def compile_scc_to_ir(
 
 def compile_modules_to_ir(
     result: BuildResult,
-    mapper: genops.Mapper,
+    mapper: Mapper,
     compiler_options: CompilerOptions,
     errors: Errors,
 ) -> ModuleIRs:
@@ -259,7 +261,7 @@ def compile_ir_to_c(
     groups: Groups,
     modules: ModuleIRs,
     result: BuildResult,
-    mapper: genops.Mapper,
+    mapper: Mapper,
     compiler_options: CompilerOptions,
 ) -> Dict[Optional[str], List[Tuple[str, str]]]:
     """Compile a collection of ModuleIRs to C source text.
@@ -336,11 +338,16 @@ def write_cache(
         st = result.graph[id]
 
         meta_path, _, _ = get_cache_names(id, st.xpath, result.manager.options)
+        # If the metadata isn't there, skip writing the cache.
+        try:
+            meta_data = result.manager.metastore.read(meta_path)
+        except IOError:
+            continue
 
         newpath = get_state_ir_cache_name(st)
         ir_data = {
             'ir': module.serialize(),
-            'meta_hash': compute_hash(result.manager.metastore.read(meta_path)),
+            'meta_hash': compute_hash(meta_data),
             'src_hashes': hashes[group_map[id]],
         }
 
@@ -352,7 +359,7 @@ def write_cache(
 def load_scc_from_cache(
     scc: List[MypyFile],
     result: BuildResult,
-    mapper: genops.Mapper,
+    mapper: Mapper,
     ctx: DeserMaps,
 ) -> ModuleIRs:
     """Load IR for an SCC of modules from the cache.
@@ -365,7 +372,7 @@ def load_scc_from_cache(
         )['ir'] for k in scc
     }
     modules = deserialize_modules(cache_data, ctx)
-    genops.load_type_map(mapper, scc, ctx)
+    load_type_map(mapper, scc, ctx)
     return modules
 
 
@@ -395,7 +402,7 @@ def compile_modules_to_c(
     """
     # Construct a map from modules to what group they belong to
     group_map = {source.module: lib_name for group, lib_name in groups for source in group}
-    mapper = genops.Mapper(group_map)
+    mapper = Mapper(group_map)
 
     modules = compile_modules_to_ir(result, mapper, compiler_options, errors)
     ctext = compile_ir_to_c(groups, modules, result, mapper, compiler_options)
@@ -419,10 +426,10 @@ def pointerize(decl: str, name: str) -> str:
     """Given a C decl and its name, modify it to be a declaration to a pointer."""
     # This doesn't work in general but does work for all our types...
     if '(' in decl:
-        # Function pointer. Stick a * in front of the name and wrap it in parens.
+        # Function pointer. Stick an * in front of the name and wrap it in parens.
         return decl.replace(name, '(*{})'.format(name))
     else:
-        # Non-function pointer. Just stick a * in front of the name.
+        # Non-function pointer. Just stick an * in front of the name.
         return decl.replace(name, '*{}'.format(name))
 
 
