@@ -147,6 +147,11 @@ static PyObject *CPyType_FromTemplate(PyTypeObject *template_,
         if (!ns)
             goto error;
 
+        if (bases != orig_bases) {
+            if (PyDict_SetItemString(ns, "__orig_bases__", orig_bases) < 0)
+                goto error;
+        }
+
         dummy_class = (PyTypeObject *)PyObject_CallFunctionObjArgs(
             (PyObject *)metaclass, name, bases, ns, NULL);
         Py_DECREF(ns);
@@ -925,6 +930,16 @@ static PyObject *CPyDict_FromAny(PyObject *obj) {
     }
 }
 
+static PyObject *CPyStr_Split(PyObject *str, PyObject *sep, CPyTagged max_split)
+{
+    Py_ssize_t temp_max_split = CPyTagged_AsSsize_t(max_split);
+    if (temp_max_split == -1 && PyErr_Occurred()) {
+        PyErr_SetString(PyExc_OverflowError, "Python int too large to convert to C ssize_t");
+            return NULL;
+    }
+    return PyUnicode_Split(str, sep, temp_max_split);
+}
+
 static PyObject *CPyIter_Next(PyObject *iter)
 {
     return (*iter->ob_type->tp_iternext)(iter);
@@ -1057,6 +1072,7 @@ static PyObject *CPy_FormatTypeName(PyObject *value) {
     return output;
 }
 
+CPy_NOINLINE
 static void CPy_TypeError(const char *expected, PyObject *value) {
     PyObject *out = CPy_FormatTypeName(value);
     if (out) {
