@@ -14,10 +14,11 @@ from mypy.test.data import DataSuite, DataDrivenTestCase
 from mypy.test.config import test_temp_dir
 from mypy.test.helpers import assert_string_arrays_equal
 
-from mypyc import genops
+from mypyc.genopsmain import build_ir
 from mypyc.options import CompilerOptions
 from mypyc.ops import FuncIR
 from mypyc.errors import Errors
+from mypyc.genopsmapper import Mapper
 from mypyc.test.config import test_data_prefix
 
 # The builtins stub used during icode generation test cases.
@@ -105,12 +106,13 @@ def build_ir_for_single_file(input_lines: List[str],
         raise CompileError(result.errors)
 
     errors = Errors()
-    _, modules = genops.build_ir(
+    modules = build_ir(
         [result.files['__main__']], result.graph, result.types,
+        Mapper({'__main__': None}),
         compiler_options, errors)
     assert errors.num_errors == 0
 
-    module = modules[0][1]
+    module = list(modules.values())[0]
     return module.functions
 
 
@@ -187,9 +189,18 @@ def heading(text: str) -> None:
     print('=' * 20 + ' ' + text + ' ' + '=' * 20)
 
 
-def show_c(cfiles: List[Tuple[str, str]]) -> None:
+def show_c(cfiles: List[List[Tuple[str, str]]]) -> None:
     heading('Generated C')
-    for cfile, ctext in cfiles:
-        print('== {} =='.format(cfile))
-        print_with_line_numbers(ctext)
+    for group in cfiles:
+        for cfile, ctext in group:
+            print('== {} =='.format(cfile))
+            print_with_line_numbers(ctext)
     heading('End C')
+
+
+def fudge_dir_mtimes(dir: str, delta: int) -> None:
+    for dirpath, _, filenames in os.walk(dir):
+        for name in filenames:
+            path = os.path.join(dirpath, name)
+            new_mtime = os.stat(path).st_mtime + delta
+            os.utime(path, times=(new_mtime, new_mtime))
